// This is an open source non-commercial project. Dear PVS-Studio, please check it.
// PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com

#include "Mine.h"


using namespace std;


/**
 * Helper function that gets the total amount of ore 
 */
static int getOreCount(const Mine::MineVeins& veins, Mine::OreType ore, size_t depth)
{
	int _value = 0;
	for (size_t i = 0; i < depth; ++i)
	{
		_value += veins[i][ore];
	}
	return _value;
}


/**
 * C'tor
 */
Mine::Mine() : mFlags("11110")
{}


/**
 * C'tor
 */
Mine::Mine(MineProductionRate rate) : mProductionRate(rate), mFlags("11110")
{}


/**
 * 
 */
bool Mine::active() const
{
	return mFlags[4];
}


/**
 * 
 */
void Mine::active(bool _b)
{
	mFlags[4] = _b;
}


/**
 * 
 */
bool Mine::miningCommonMetals() const
{
	return mFlags[ORE_COMMON_METALS];
}


/**
 * 
 */
bool Mine::miningCommonMinerals() const
{
	return mFlags[ORE_COMMON_MINERALS];
}


/**
 * 
 */
bool Mine::miningRareMetals() const
{
	return mFlags[ORE_RARE_METALS];
}


/**
 * 
 */
bool Mine::miningRareMinerals() const
{
	return mFlags[ORE_RARE_MINERALS];
}


/**
 * 
 */
void Mine::miningCommonMetals(bool _b)
{
	mFlags[ORE_COMMON_METALS] = _b;
}


/**
 * 
 */
void Mine::miningCommonMinerals(bool _b)
{
	mFlags[ORE_COMMON_MINERALS] = _b;
}


/**
 * 
 */
void Mine::miningRareMetals(bool _b)
{
	mFlags[ORE_RARE_METALS] = _b;
}


/**
 * 
 */
void Mine::miningRareMinerals(bool _b)
{
	mFlags[ORE_RARE_MINERALS] = _b;
}


/**
 * Increases the depth of the mine.
 * 
 * \note	This function only modifies the Mine. It has no knowledge of the
 *			maximum digging depth of a planet and doesn't modify any tiles.
 */
void Mine::increaseDepth()
{
	MineVein vein;

	switch (productionRate())
	{
	case PRODUCTION_RATE_LOW:
		vein[ORE_COMMON_METALS] = 150;
		vein[ORE_COMMON_MINERALS] = 125;
		vein[ORE_RARE_METALS] = 150;
		vein[ORE_RARE_MINERALS] = 125;
		break;

	case PRODUCTION_RATE_MEDIUM:
		vein[ORE_COMMON_METALS] = 200;
		vein[ORE_COMMON_MINERALS] = 175;
		vein[ORE_RARE_METALS] = 200;
		vein[ORE_RARE_MINERALS] = 175;
		break;

	case PRODUCTION_RATE_HIGH:
		vein[ORE_COMMON_METALS] = 250;
		vein[ORE_COMMON_MINERALS] = 200;
		vein[ORE_RARE_METALS] = 250;
		vein[ORE_RARE_MINERALS] = 200;
		break;
	}

	mVeins.push_back(vein);
}


/**
 * Gets the current depth of the mine.
 */
int Mine::depth() const
{
	return mVeins.size();
}


/**
 * Convenience function that gets the common metal ore available
 * given the current level.
 */
int Mine::commonMetalsAvailable() const
{
	return getOreCount(mVeins, ORE_COMMON_METALS, depth());
}


/**
 * Convenience function that gets the common mineral ore available
 * given the current level.
 */
int Mine::commonMineralsAvailable() const
{
	return getOreCount(mVeins, ORE_COMMON_MINERALS, depth());
}


/**
 * Convenience function that gets the rare metal ore available
 * given the current level.
 */
int Mine::rareMetalsAvailable() const
{
	return getOreCount(mVeins, ORE_RARE_METALS, depth());
}


/**
 * Convenience function that gets the rare mineral ore available
 * given the current level.
 */
int Mine::rareMineralsAvailable() const
{
	return getOreCount(mVeins, ORE_RARE_MINERALS, depth());
}


/**
 * Indicates that there are no resources available at this mine.
 */
bool Mine::exhausted() const
{
	int ore_count = 0;

	for (auto vein : mVeins)
	{
		ore_count += vein[ORE_COMMON_METALS];
		ore_count += vein[ORE_COMMON_MINERALS];
		ore_count += vein[ORE_RARE_METALS];
		ore_count += vein[ORE_RARE_MINERALS];
	}

	return (ore_count == 0);
}


/**
 * Pulls the specified quantity of Ore from the Mine. If
 * insufficient ore is available, only pulls what's available.
 */
int Mine::pull(OreType type, int quantity)
{
	int pulled_count = 0, to_pull = quantity;
	
	for (size_t i = 0; i < mVeins.size(); ++i)
	{
		MineVein& vein = mVeins[i];

		if (vein[type] >= to_pull)
		{
			pulled_count = to_pull;
			vein[type] -= to_pull;
			break;
		}
		else if (vein[type] < to_pull)
		{
			pulled_count += vein[type];
			to_pull = to_pull - vein[type];
			vein[type] = 0;
		}
	}

	return pulled_count;
}
